@@ -85,7 +85,7 @@ def fromstring(text, root_tag=None, logger=None):
     return elem
 
 
-def tostring(tree, use_s3ns=True):
+def tostring(tree, use_s3ns=True, xml_declaration=True):
     if use_s3ns:
         nsmap = tree.nsmap.copy()
         nsmap[None] = XMLNS_S3
@@ -95,7 +95,8 @@ def tostring(tree, use_s3ns=True):
         root.extend(deepcopy(tree.getchildren()))
         tree = root
 
-    return lxml.etree.tostring(tree, xml_declaration=True, encoding='UTF-8')
+    return lxml.etree.tostring(tree, xml_declaration=xml_declaration,
+                               encoding='UTF-8')
 
 
 class _Element(lxml.etree.ElementBase):
