@@ -430,20 +430,21 @@ class SigV4Mixin(object):
             raise InvalidRequest(msg)
         else:
             hashed_payload = self.headers['X-Amz-Content-SHA256']
-            if self.content_length == 0:
-                if hashed_payload != sha256().hexdigest():
-                    raise BadDigest(
-                        'The X-Amz-Content-SHA56 you specified did not match '
-                        'what we received.')
-            elif self.content_length:
-                self.environ['wsgi.input'] = HashingInput(
-                    self.environ['wsgi.input'],
-                    self.content_length,
-                    sha256,
-                    hashed_payload)
-            # else, not provided -- Swift will kick out a 411 Length Required
-            # which will get translated back to a S3-style response in
-            # S3Request._swift_error_codes
+            if hashed_payload != 'UNSIGNED-PAYLOAD':
+                if self.content_length == 0:
+                    if hashed_payload != sha256().hexdigest():
+                        raise BadDigest(
+                            'The X-Amz-Content-SHA56 you specified did not '
+                            'match what we received.')
+                elif self.content_length:
+                    self.environ['wsgi.input'] = HashingInput(
+                        self.environ['wsgi.input'],
+                        self.content_length,
+                        sha256,
+                        hashed_payload)
+                # else, length not provided -- Swift will kick out a
+                # 411 Length Required which will get translated back
+                # to a S3-style response in S3Request._swift_error_codes
         cr.append(hashed_payload)
         return '\n'.join(cr).encode('utf-8')
 
@@ -1099,14 +1100,13 @@ class S3Request(swob.Request):
             env['HTTP_X_COPY_FROM'] = env['HTTP_X_AMZ_COPY_SOURCE']
             del env['HTTP_X_AMZ_COPY_SOURCE']
             env['CONTENT_LENGTH'] = '0'
-            # Content type cannot be modified on COPY
-            env.pop('CONTENT_TYPE', None)
             if env.pop('HTTP_X_AMZ_METADATA_DIRECTIVE', None) == 'REPLACE':
                 env['HTTP_X_FRESH_METADATA'] = 'True'
             else:
                 copy_exclude_headers = ('HTTP_CONTENT_DISPOSITION',
                                         'HTTP_CONTENT_ENCODING',
                                         'HTTP_CONTENT_LANGUAGE',
+                                        'CONTENT_TYPE',
                                         'HTTP_EXPIRES',
                                         'HTTP_CACHE_CONTROL',
                                         'HTTP_X_ROBOTS_TAG')
