@@ -32,13 +32,10 @@ from eventlet.green import socket, ssl, os as green_os
 import six
 from six import BytesIO
 from six import StringIO
-from six.moves.urllib.parse import unquote
-if six.PY2:
-    import mimetools
 
 from swift.common import utils, constraints
 from swift.common.storage_policy import BindPortsCache
-from swift.common.swob import Request
+from swift.common.swob import Request, wsgi_unquote
 from swift.common.utils import capture_stdio, disable_fallocate, \
     drop_privileges, get_logger, NullLogger, config_true_value, \
     validate_configuration, get_hub, config_auto_int_value, \
@@ -148,31 +145,6 @@ def wrap_conf_type(f):
 appconfig = wrap_conf_type(loadwsgi.appconfig)
 
 
-def monkey_patch_mimetools():
-    """
-    mimetools.Message defaults content-type to "text/plain"
-    This changes it to default to None, so we can detect missing headers.
-    """
-    if six.PY3:
-        # The mimetools has been removed from Python 3
-        return
-
-    orig_parsetype = mimetools.Message.parsetype
-
-    def parsetype(self):
-        if not self.typeheader:
-            self.type = None
-            self.maintype = None
-            self.subtype = None
-            self.plisttext = ''
-        else:
-            orig_parsetype(self)
-    parsetype.patched = True
-
-    if not getattr(mimetools.Message.parsetype, 'patched', None):
-        mimetools.Message.parsetype = parsetype
-
-
 def get_socket(conf):
     """Bind socket to bind ip:port in conf
 
@@ -448,6 +420,18 @@ class SwiftHttpProtocol(wsgi.HttpProtocol):
             # versions the output from error is same as info anyway
             self.server.log.info('ERROR WSGI: ' + f, *a)
 
+    class MessageClass(wsgi.HttpProtocol.MessageClass):
+        '''Subclass to see when the client didn't provide a Content-Type'''
+        # for py2:
+        def parsetype(self):
+            if self.typeheader is None:
+                self.typeheader = ''
+            wsgi.HttpProtocol.MessageClass.parsetype(self)
+
+        # for py3:
+        def get_default_type(self):
+            return ''
+
 
 class SwiftHttpProxiedProtocol(SwiftHttpProtocol):
     """
@@ -1156,7 +1140,6 @@ def _initrp(conf_path, app_section, *args, **kwargs):
     if config_true_value(conf.get('disable_fallocate', 'no')):
         disable_fallocate()
 
-    monkey_patch_mimetools()
     return (conf, logger, log_name)
 
 
@@ -1319,7 +1302,7 @@ def make_subrequest(env, method=None, path=None, body=None, headers=None,
     path = path or ''
     if path and '?' in path:
         path, query_string = path.split('?', 1)
-    newenv = make_env(env, method, path=unquote(path), agent=agent,
+    newenv = make_env(env, method, path=wsgi_unquote(path), agent=agent,
                       query_string=query_string, swift_source=swift_source)
     if not headers:
         headers = {}
