@@ -53,8 +53,7 @@ from test.unit import SkipTest
 
 from swift.common import constraints, utils, ring, storage_policy
 from swift.common.ring import Ring
-from swift.common.wsgi import (
-    monkey_patch_mimetools, loadapp, SwiftHttpProtocol)
+from swift.common.wsgi import loadapp, SwiftHttpProtocol
 from swift.common.utils import config_true_value, split_path
 from swift.account import server as account_server
 from swift.container import server as container_server
@@ -150,7 +149,7 @@ def _in_process_setup_swift_conf(swift_conf_src, testdir):
         conf.set(section, 'swift_hash_path_prefix', 'inprocfunctests')
         section = 'swift-constraints'
         max_file_size = (8 * 1024 * 1024) + 2  # 8 MB + 2
-        conf.set(section, 'max_file_size', max_file_size)
+        conf.set(section, 'max_file_size', str(max_file_size))
     except NoSectionError:
         msg = 'Conf file %s is missing section %s' % (swift_conf_src, section)
         raise InProcessException(msg)
@@ -232,8 +231,8 @@ def _in_process_setup_ring(swift_conf, conf_src_dir, testdir):
     sp_zero_section = sp_prefix + '0'
     conf.add_section(sp_zero_section)
     for (k, v) in policy_to_test.get_info(config=True).items():
-        conf.set(sp_zero_section, k, v)
-    conf.set(sp_zero_section, 'default', True)
+        conf.set(sp_zero_section, k, str(v))
+    conf.set(sp_zero_section, 'default', 'True')
 
     with open(swift_conf, 'w') as fp:
         conf.write(fp)
@@ -493,8 +492,6 @@ def in_process_setup(the_object_server=object_server):
     swift_conf_src = _in_process_find_conf_file(conf_src_dir, 'swift.conf')
     _info('Using swift config from %s' % swift_conf_src)
 
-    monkey_patch_mimetools()
-
     global _testdir
     _testdir = os.path.join(mkdtemp(), 'tmp_functional')
     utils.mkdirs(_testdir)
@@ -714,7 +711,7 @@ def in_process_setup(the_object_server=object_server):
                 '/' + act, {'X-Timestamp': ts, 'x-trans-id': act})
             resp = conn.getresponse()
             assert resp.status == 201, 'Unable to create account: %s\n%s' % (
-                resp.status, resp.body)
+                resp.status, resp.read())
 
     create_account('AUTH_test')
     create_account('AUTH_test2')
@@ -1292,3 +1289,15 @@ def requires_policies(f):
         return f(self, *args, **kwargs)
 
     return wrapper
+
+
+def requires_bulk(f):
+    @functools.wraps(f)
+    def wrapper(*args, **kwargs):
+        if skip or not cluster_info:
+            raise SkipTest('Requires bulk middleware')
+        # Determine whether this cluster has bulk middleware; if not, skip test
+        if not cluster_info.get('bulk_upload', {}):
+            raise SkipTest('Requires bulk middleware')
+        return f(*args, **kwargs)
+    return wrapper
