@@ -520,9 +520,37 @@ class TestS3ApiObj(S3ApiTestCase):
         req.content_type = 'text/plain'
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '400')
-        print(body)
         self.assertEqual(self._get_error_code(body), 'BadDigest')
 
+    @s3acl
+    def test_object_PUT_v4_unsigned_payload(self):
+        req = Request.blank(
+            '/bucket/object',
+            environ={'REQUEST_METHOD': 'PUT'},
+            headers={
+                'Authorization':
+                    'AWS4-HMAC-SHA256 '
+                    'Credential=test:tester/%s/us-east-1/s3/aws4_request, '
+                    'SignedHeaders=host;x-amz-date, '
+                    'Signature=hmac' % (
+                        self.get_v4_amz_date_header().split('T', 1)[0]),
+                'x-amz-date': self.get_v4_amz_date_header(),
+                'x-amz-storage-class': 'STANDARD',
+                'x-amz-content-sha256': 'UNSIGNED-PAYLOAD',
+                'Date': self.get_date_header()},
+            body=self.object_body)
+        req.date = datetime.now()
+        req.content_type = 'text/plain'
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status.split()[0], '200')
+        # Check that s3api returns an etag header.
+        self.assertEqual(headers['etag'],
+                         '"%s"' % self.response_headers['etag'])
+
+        _, _, headers = self.swift.calls_with_headers[-1]
+        # No way to determine ETag to send
+        self.assertNotIn('etag', headers)
+
     def test_object_PUT_headers(self):
         content_md5 = self.etag.decode('hex').encode('base64').strip()
 
@@ -689,8 +717,7 @@ class TestS3ApiObj(S3ApiTestCase):
         self.assertEqual(headers['Content-Disposition'], 'how are you')
         self.assertEqual(headers['Content-Encoding'], 'good and you')
         self.assertEqual(headers['Content-Language'], 'great')
-        # Content-Type can't be set during an S3 copy operation
-        self.assertIsNone(headers.get('Content-Type'))
+        self.assertEqual(headers['Content-Type'], 'so')
         self.assertEqual(headers['Expires'], 'yeah')
         self.assertEqual(headers['X-Robots-Tag'], 'bye')
 
@@ -1179,5 +1206,6 @@ class TestS3ApiObjNonUTC(TestS3ApiObj):
         os.environ['TZ'] = self.orig_tz
         time.tzset()
 
+
 if __name__ == '__main__':
     unittest.main()
