@@ -504,6 +504,98 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
         self._assert_authorized(req, account_path='/v1/')
         self.assertEqual(req.environ['PATH_INFO'], '/v1/AUTH_TENANT_ID/c/o')
 
+    @mock.patch('swift.common.middleware.s3api.s3token.cache_from_env')
+    @mock.patch('keystoneclient.v3.client.Client')
+    @mock.patch.object(requests, 'post')
+    def test_secret_is_cached(self, MOCK_REQUEST, MOCK_KEYSTONE,
+                              MOCK_CACHE_FROM_ENV):
+        self.middleware = s3token.filter_factory({
+            'auth_uri': 'http://example.com',
+            'secret_cache_duration': '20',
+            'auth_type': 'v3password',
+            'auth_url': 'http://example.com:5000/v3',
+            'username': 'swift',
+            'password': 'secret',
+            'project_name': 'service',
+            'user_domain_name': 'default',
+            'project_domain_name': 'default',
+        })(FakeApp())
+        self.assertEqual(20, self.middleware._secret_cache_duration)
+
+        cache = MOCK_CACHE_FROM_ENV.return_value
+
+        fake_cache_response = ({}, 'token_id', {'id': 'tenant_id'}, 'secret')
+        cache.get.return_value = fake_cache_response
+
+        MOCK_REQUEST.return_value = TestResponse({
+            'status_code': 201,
+            'text': json.dumps(GOOD_RESPONSE_V2)})
+
+        req = Request.blank('/v1/AUTH_cfa/c/o')
+        req.environ['s3api.auth_details'] = {
+            'access_key': u'access',
+            'signature': u'signature',
+            'string_to_sign': u'token',
+            'check_signature': lambda x: True
+        }
+        req.get_response(self.middleware)
+        # Ensure we don't request auth from keystone
+        self.assertFalse(MOCK_REQUEST.called)
+
+    @mock.patch('swift.common.middleware.s3api.s3token.cache_from_env')
+    @mock.patch('keystoneclient.v3.client.Client')
+    @mock.patch.object(requests, 'post')
+    def test_secret_sets_cache(self, MOCK_REQUEST, MOCK_KEYSTONE,
+                               MOCK_CACHE_FROM_ENV):
+        self.middleware = s3token.filter_factory({
+            'auth_uri': 'http://example.com',
+            'secret_cache_duration': '20',
+            'auth_type': 'v3password',
+            'auth_url': 'http://example.com:5000/v3',
+            'username': 'swift',
+            'password': 'secret',
+            'project_name': 'service',
+            'user_domain_name': 'default',
+            'project_domain_name': 'default',
+        })(FakeApp())
+        self.assertEqual(20, self.middleware._secret_cache_duration)
+
+        cache = MOCK_CACHE_FROM_ENV.return_value
+        cache.get.return_value = None
+
+        keystone_client = MOCK_KEYSTONE.return_value
+        keystone_client.ec2.get.return_value = mock.Mock(secret='secret')
+
+        MOCK_REQUEST.return_value = TestResponse({
+            'status_code': 201,
+            'text': json.dumps(GOOD_RESPONSE_V2)})
+
+        req = Request.blank('/v1/AUTH_cfa/c/o')
+        req.environ['s3api.auth_details'] = {
+            'access_key': u'access',
+            'signature': u'signature',
+            'string_to_sign': u'token',
+            'check_signature': lambda x: True
+        }
+        req.get_response(self.middleware)
+        expected_headers = {
+            'X-Identity-Status': u'Confirmed',
+            'X-Roles': u'swift-user,_member_',
+            'X-User-Id': u'USER_ID',
+            'X-User-Name': u'S3_USER',
+            'X-Tenant-Id': u'TENANT_ID',
+            'X-Tenant-Name': u'TENANT_NAME',
+            'X-Project-Id': u'TENANT_ID',
+            'X-Project-Name': u'TENANT_NAME',
+        }
+
+        self.assertTrue(MOCK_REQUEST.called)
+        tenant = GOOD_RESPONSE_V2['access']['token']['tenant']
+        token = GOOD_RESPONSE_V2['access']['token']['id']
+        expected_cache = (expected_headers, token, tenant, 'secret')
+        cache.set.assert_called_once_with('s3secret/access', expected_cache,
+                                          time=20)
+
 
 class S3TokenMiddlewareTestBad(S3TokenMiddlewareTestBase):
     def test_unauthorized_token(self):
