@@ -72,7 +72,7 @@ class FakeConn(object):
         return self.resp_headers.items()
 
     def read(self, amt=None):
-        if isinstance(self.body, six.StringIO):
+        if isinstance(self.body, six.BytesIO):
             return self.body.read(amt)
         elif amt is None:
             return self.body
@@ -117,28 +117,61 @@ class TestDirectClient(unittest.TestCase):
                 self.account, self.container, self.obj))
         self.user_agent = 'direct-client %s' % os.getpid()
 
+        class FakeTimeout(BaseException):
+            def __enter__(self):
+                return self
+
+            def __exit__(self, typ, value, tb):
+                pass
+
+        patcher = mock.patch.object(direct_client, 'Timeout', FakeTimeout)
+        patcher.start()
+        self.addCleanup(patcher.stop)
+
     def test_gen_headers(self):
         stub_user_agent = 'direct-client %s' % os.getpid()
 
-        headers = direct_client.gen_headers()
+        headers = direct_client.gen_headers(add_ts=False)
         self.assertEqual(headers['user-agent'], stub_user_agent)
         self.assertEqual(1, len(headers))
 
         now = time.time()
-        headers = direct_client.gen_headers(add_ts=True)
+        headers = direct_client.gen_headers()
         self.assertEqual(headers['user-agent'], stub_user_agent)
         self.assertTrue(now - 1 < Timestamp(headers['x-timestamp']) < now + 1)
         self.assertEqual(headers['x-timestamp'],
                          Timestamp(headers['x-timestamp']).internal)
         self.assertEqual(2, len(headers))
 
-        headers = direct_client.gen_headers(hdrs_in={'foo-bar': '47'})
+        headers = direct_client.gen_headers(hdrs_in={'x-timestamp': '15'})
+        self.assertEqual(headers['x-timestamp'], '15')
         self.assertEqual(headers['user-agent'], stub_user_agent)
-        self.assertEqual(headers['foo-bar'], '47')
         self.assertEqual(2, len(headers))
 
-        headers = direct_client.gen_headers(hdrs_in={'user-agent': '47'})
+        headers = direct_client.gen_headers(hdrs_in={'foo-bar': '63'})
         self.assertEqual(headers['user-agent'], stub_user_agent)
+        self.assertEqual(headers['foo-bar'], '63')
+        self.assertTrue(now - 1 < Timestamp(headers['x-timestamp']) < now + 1)
+        self.assertEqual(headers['x-timestamp'],
+                         Timestamp(headers['x-timestamp']).internal)
+        self.assertEqual(3, len(headers))
+
+        hdrs_in = {'foo-bar': '55'}
+        headers = direct_client.gen_headers(hdrs_in, add_ts=False)
+        self.assertEqual(headers['user-agent'], stub_user_agent)
+        self.assertEqual(headers['foo-bar'], '55')
+        self.assertEqual(2, len(headers))
+
+        headers = direct_client.gen_headers(hdrs_in={'user-agent': '32'})
+        self.assertEqual(headers['user-agent'], '32')
+        self.assertTrue(now - 1 < Timestamp(headers['x-timestamp']) < now + 1)
+        self.assertEqual(headers['x-timestamp'],
+                         Timestamp(headers['x-timestamp']).internal)
+        self.assertEqual(2, len(headers))
+
+        hdrs_in = {'user-agent': '47'}
+        headers = direct_client.gen_headers(hdrs_in, add_ts=False)
+        self.assertEqual(headers['user-agent'], '47')
         self.assertEqual(1, len(headers))
 
         for policy in POLICIES:
@@ -169,7 +202,7 @@ class TestDirectClient(unittest.TestCase):
                 'X-Timestamp': '1234567890',
                 'X-PUT-Timestamp': '1234567890'})
 
-            body = '[{"count": 1, "bytes": 20971520, "name": "c1"}]'
+            body = b'[{"count": 1, "bytes": 20971520, "name": "c1"}]'
 
             with mocked_http_conn(200, stub_headers, body) as conn:
                 resp_headers, resp = direct_client.direct_get_account(
@@ -207,28 +240,23 @@ class TestDirectClient(unittest.TestCase):
         stub_headers = {'X-Trans-Id': 'txb5f59485c578460f8be9e-0053478d09'}
         body = 'a server error has occurred'
         with mocked_http_conn(500, stub_headers, body):
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_get_account(self.node, self.part,
                                                  self.account)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
-        self.assertEqual(err.http_status, 500)
+        self.assertEqual(raised.exception.http_status, 500)
         expected_err_msg_parts = (
             'Account server %s:%s' % (self.node['ip'], self.node['port']),
             'GET %r' % self.account_path,
             'status 500',
         )
         for item in expected_err_msg_parts:
-            self.assertTrue(
-                item in str(err), '%r was not in "%s"' % (item, err))
-        self.assertEqual(err.http_host, self.node['ip'])
-        self.assertEqual(err.http_port, self.node['port'])
-        self.assertEqual(err.http_device, self.node['device'])
-        self.assertEqual(err.http_status, 500)
-        self.assertEqual(err.http_reason, 'Internal Error')
-        self.assertEqual(err.http_headers, stub_headers)
+            self.assertIn(item, str(raised.exception))
+        self.assertEqual(raised.exception.http_host, self.node['ip'])
+        self.assertEqual(raised.exception.http_port, self.node['port'])
+        self.assertEqual(raised.exception.http_device, self.node['device'])
+        self.assertEqual(raised.exception.http_status, 500)
+        self.assertEqual(raised.exception.http_reason, 'Internal Error')
+        self.assertEqual(raised.exception.http_headers, stub_headers)
 
     def test_direct_get_account_no_content_does_not_parse_body(self):
         headers = {
@@ -249,17 +277,13 @@ class TestDirectClient(unittest.TestCase):
 
     def test_direct_get_account_error(self):
         with mocked_http_conn(500) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_get_account(
                     self.node, self.part, self.account)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
             self.assertEqual(conn.method, 'GET')
             self.assertEqual(conn.path, self.account_path)
-        self.assertEqual(err.http_status, 500)
-        self.assertTrue('GET' in str(err))
+        self.assertEqual(raised.exception.http_status, 500)
+        self.assertTrue('GET' in str(raised.exception))
 
     def test_direct_delete_account(self):
         part = '0'
@@ -287,17 +311,15 @@ class TestDirectClient(unittest.TestCase):
         account = 'a'
 
         with mocked_http_conn(500) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_delete_account(self.node, part, account)
-            except ClientException as err:
-                pass
             self.assertEqual(self.node['ip'], conn.host)
             self.assertEqual(self.node['port'], conn.port)
             self.assertEqual('DELETE', conn.method)
             self.assertEqual('/sda/0/a', conn.path)
             self.assertIn('X-Timestamp', conn.req_headers)
             self.assertIn('User-Agent', conn.req_headers)
-            self.assertEqual(err.http_status, 500)
+            self.assertEqual(raised.exception.http_status, 500)
 
     def test_direct_head_container(self):
         headers = HeaderKeyDict(key='value')
@@ -318,13 +340,9 @@ class TestDirectClient(unittest.TestCase):
         headers = HeaderKeyDict(key='value')
 
         with mocked_http_conn(503, headers) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_head_container(
                     self.node, self.part, self.account, self.container)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
             # check request
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
@@ -332,9 +350,9 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.path, self.container_path)
 
         self.assertEqual(conn.req_headers['user-agent'], self.user_agent)
-        self.assertEqual(err.http_status, 503)
-        self.assertEqual(err.http_headers, headers)
-        self.assertTrue('HEAD' in str(err))
+        self.assertEqual(raised.exception.http_status, 503)
+        self.assertEqual(raised.exception.http_headers, headers)
+        self.assertTrue('HEAD' in str(raised.exception))
 
     def test_direct_head_container_deleted(self):
         important_timestamp = Timestamp.now().internal
@@ -342,27 +360,23 @@ class TestDirectClient(unittest.TestCase):
                                  important_timestamp})
 
         with mocked_http_conn(404, headers) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_head_container(
                     self.node, self.part, self.account, self.container)
-            except Exception as err:
-                self.assertTrue(isinstance(err, ClientException))
-            else:
-                self.fail('ClientException not raised')
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'HEAD')
             self.assertEqual(conn.path, self.container_path)
 
         self.assertEqual(conn.req_headers['user-agent'], self.user_agent)
-        self.assertEqual(err.http_status, 404)
-        self.assertEqual(err.http_headers, headers)
+        self.assertEqual(raised.exception.http_status, 404)
+        self.assertEqual(raised.exception.http_headers, headers)
 
     def test_direct_get_container(self):
         def do_test(req_params):
             headers = HeaderKeyDict({'key': 'value'})
-            body = ('[{"hash": "8f4e3", "last_modified": "317260", '
-                    '"bytes": 209}]')
+            body = (b'[{"hash": "8f4e3", "last_modified": "317260", '
+                    b'"bytes": 209}]')
 
             with mocked_http_conn(200, headers, body) as conn:
                 resp_headers, resp = direct_client.direct_get_container(
@@ -434,21 +448,78 @@ class TestDirectClient(unittest.TestCase):
 
     def test_direct_delete_container_error(self):
         with mocked_http_conn(500) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_delete_container(
                     self.node, self.part, self.account, self.container)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
 
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'DELETE')
             self.assertEqual(conn.path, self.container_path)
 
-        self.assertEqual(err.http_status, 500)
-        self.assertTrue('DELETE' in str(err))
+        self.assertEqual(raised.exception.http_status, 500)
+        self.assertTrue('DELETE' in str(raised.exception))
+
+    def test_direct_put_container(self):
+        body = b'Let us begin with a quick introduction'
+        headers = {'x-foo': 'bar', 'Content-Length': str(len(body)),
+                   'Content-Type': 'application/json',
+                   'User-Agent': 'my UA'}
+
+        with mocked_http_conn(204) as conn:
+            rv = direct_client.direct_put_container(
+                self.node, self.part, self.account, self.container,
+                contents=body, headers=headers)
+            self.assertEqual(conn.host, self.node['ip'])
+            self.assertEqual(conn.port, self.node['port'])
+            self.assertEqual(conn.method, 'PUT')
+            self.assertEqual(conn.path, self.container_path)
+            self.assertEqual(conn.req_headers['Content-Length'],
+                             str(len(body)))
+            self.assertEqual(conn.req_headers['Content-Type'],
+                             'application/json')
+            self.assertEqual(conn.req_headers['User-Agent'], 'my UA')
+            self.assertTrue('x-timestamp' in conn.req_headers)
+            self.assertEqual('bar', conn.req_headers.get('x-foo'))
+            self.assertEqual(md5(body).hexdigest(), conn.etag.hexdigest())
+        self.assertIsNone(rv)
+
+    def test_direct_put_container_chunked(self):
+        body = b'Let us begin with a quick introduction'
+        headers = {'x-foo': 'bar', 'Content-Type': 'application/json'}
+
+        with mocked_http_conn(204) as conn:
+            rv = direct_client.direct_put_container(
+                self.node, self.part, self.account, self.container,
+                contents=body, headers=headers)
+            self.assertEqual(conn.host, self.node['ip'])
+            self.assertEqual(conn.port, self.node['port'])
+            self.assertEqual(conn.method, 'PUT')
+            self.assertEqual(conn.path, self.container_path)
+            self.assertEqual(conn.req_headers['Transfer-Encoding'], 'chunked')
+            self.assertEqual(conn.req_headers['Content-Type'],
+                             'application/json')
+            self.assertTrue('x-timestamp' in conn.req_headers)
+            self.assertEqual('bar', conn.req_headers.get('x-foo'))
+            self.assertNotIn('Content-Length', conn.req_headers)
+            expected_sent = b'%0x\r\n%s\r\n0\r\n\r\n' % (len(body), body)
+            self.assertEqual(md5(expected_sent).hexdigest(),
+                             conn.etag.hexdigest())
+        self.assertIsNone(rv)
+
+    def test_direct_put_container_fail(self):
+        with mock.patch('swift.common.bufferedhttp.http_connect_raw',
+                        side_effect=Exception('conn failed')):
+            with self.assertRaises(Exception) as cm:
+                direct_client.direct_put_container(
+                    self.node, self.part, self.account, self.container)
+        self.assertEqual('conn failed', str(cm.exception))
+
+        with mocked_http_conn(Exception('resp failed')):
+            with self.assertRaises(Exception) as cm:
+                direct_client.direct_put_container(
+                    self.node, self.part, self.account, self.container)
+        self.assertEqual('resp failed', str(cm.exception))
 
     def test_direct_put_container_object(self):
         headers = {'x-foo': 'bar'}
@@ -468,22 +539,18 @@ class TestDirectClient(unittest.TestCase):
 
     def test_direct_put_container_object_error(self):
         with mocked_http_conn(500) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_put_container_object(
                     self.node, self.part, self.account, self.container,
                     self.obj)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
 
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'PUT')
             self.assertEqual(conn.path, self.obj_path)
 
-        self.assertEqual(err.http_status, 500)
-        self.assertTrue('PUT' in str(err))
+        self.assertEqual(raised.exception.http_status, 500)
+        self.assertTrue('PUT' in str(raised.exception))
 
     def test_direct_delete_container_object(self):
         with mocked_http_conn(204) as conn:
@@ -498,22 +565,18 @@ class TestDirectClient(unittest.TestCase):
 
     def test_direct_delete_container_obj_error(self):
         with mocked_http_conn(500) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_delete_container_object(
                     self.node, self.part, self.account, self.container,
                     self.obj)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
 
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'DELETE')
             self.assertEqual(conn.path, self.obj_path)
 
-        self.assertEqual(err.http_status, 500)
-        self.assertTrue('DELETE' in str(err))
+        self.assertEqual(raised.exception.http_status, 500)
+        self.assertTrue('DELETE' in str(raised.exception))
 
     def test_direct_head_object(self):
         headers = HeaderKeyDict({'x-foo': 'bar'})
@@ -529,50 +592,43 @@ class TestDirectClient(unittest.TestCase):
 
         self.assertEqual(conn.req_headers['user-agent'], self.user_agent)
         self.assertEqual('bar', conn.req_headers.get('x-foo'))
-        self.assertNotIn('x-timestamp', conn.req_headers)
+        self.assertIn('x-timestamp', conn.req_headers)
         self.assertEqual(headers, resp)
 
     def test_direct_head_object_error(self):
         with mocked_http_conn(500) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_head_object(
                     self.node, self.part, self.account, self.container,
                     self.obj)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'HEAD')
             self.assertEqual(conn.path, self.obj_path)
 
-        self.assertEqual(err.http_status, 500)
-        self.assertTrue('HEAD' in str(err))
+        self.assertEqual(raised.exception.http_status, 500)
+        self.assertTrue('HEAD' in str(raised.exception))
 
     def test_direct_head_object_not_found(self):
         important_timestamp = Timestamp.now().internal
         stub_headers = {'X-Backend-Important-Timestamp': important_timestamp}
         with mocked_http_conn(404, headers=stub_headers) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_head_object(
                     self.node, self.part, self.account, self.container,
                     self.obj)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'HEAD')
             self.assertEqual(conn.path, self.obj_path)
 
-        self.assertEqual(err.http_status, 404)
-        self.assertEqual(err.http_headers['x-backend-important-timestamp'],
-                         important_timestamp)
+        self.assertEqual(raised.exception.http_status, 404)
+        self.assertEqual(
+            raised.exception.http_headers['x-backend-important-timestamp'],
+            important_timestamp)
 
     def test_direct_get_object(self):
-        contents = six.StringIO('123456')
+        contents = six.BytesIO(b'123456')
 
         with mocked_http_conn(200, body=contents) as conn:
             resp_header, obj_body = direct_client.direct_get_object(
@@ -585,41 +641,30 @@ class TestDirectClient(unittest.TestCase):
 
     def test_direct_get_object_error(self):
         with mocked_http_conn(500) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_get_object(
                     self.node, self.part,
                     self.account, self.container, self.obj)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'GET')
             self.assertEqual(conn.path, self.obj_path)
 
-        self.assertEqual(err.http_status, 500)
-        self.assertTrue('GET' in str(err))
+        self.assertEqual(raised.exception.http_status, 500)
+        self.assertTrue('GET' in str(raised.exception))
 
     def test_direct_get_object_chunks(self):
-        contents = six.StringIO('123456')
-        downloaded = b''
+        contents = six.BytesIO(b'123456')
 
         with mocked_http_conn(200, body=contents) as conn:
             resp_header, obj_body = direct_client.direct_get_object(
                 self.node, self.part, self.account, self.container, self.obj,
                 resp_chunk_size=2)
-            while obj_body:
-                try:
-                    chunk = obj_body.next()
-                except StopIteration:
-                    break
-                downloaded += chunk
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual('GET', conn.method)
             self.assertEqual(self.obj_path, conn.path)
-        self.assertEqual('123456', downloaded)
+            self.assertEqual([b'12', b'34', b'56'], list(obj_body))
 
     def test_direct_post_object(self):
         headers = {'Key': 'value'}
@@ -642,14 +687,10 @@ class TestDirectClient(unittest.TestCase):
         headers = {'Key': 'value'}
 
         with mocked_http_conn(500) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_post_object(
                     self.node, self.part, self.account, self.container,
                     self.obj, headers)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'POST')
@@ -659,8 +700,8 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.req_headers['user-agent'], self.user_agent)
             self.assertTrue('x-timestamp' in conn.req_headers)
 
-        self.assertEqual(err.http_status, 500)
-        self.assertTrue('POST' in str(err))
+        self.assertEqual(raised.exception.http_status, 500)
+        self.assertTrue('POST' in str(raised.exception))
 
     def test_direct_delete_object(self):
         with mocked_http_conn(200) as conn:
@@ -689,18 +730,14 @@ class TestDirectClient(unittest.TestCase):
 
     def test_direct_delete_object_error(self):
         with mocked_http_conn(503) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_delete_object(
                     self.node, self.part, self.account, self.container,
                     self.obj)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
             self.assertEqual(conn.method, 'DELETE')
             self.assertEqual(conn.path, self.obj_path)
-        self.assertEqual(err.http_status, 503)
-        self.assertTrue('DELETE' in str(err))
+        self.assertEqual(raised.exception.http_status, 503)
+        self.assertTrue('DELETE' in str(raised.exception))
 
     def test_direct_get_suffix_hashes(self):
         data = {'a83': 'c130a2c17ed45102aada0f4eee69494ff'}
@@ -719,12 +756,12 @@ class TestDirectClient(unittest.TestCase):
             with self.assertRaises(DirectClientException) as cm:
                 direct_client.direct_get_suffix_hashes(
                     self.node, self.part, ['a83', 'b52'])
-        self.assertIn('REPLICATE', cm.exception.message)
+        self.assertIn('REPLICATE', cm.exception.args[0])
         self.assertIn(quote('/%s/%s/a83-b52'
                             % (self.node['device'], self.part)),
-                      cm.exception.message)
-        self.assertIn(self.node['replication_ip'], cm.exception.message)
-        self.assertIn(self.node['replication_port'], cm.exception.message)
+                      cm.exception.args[0])
+        self.assertIn(self.node['replication_ip'], cm.exception.args[0])
+        self.assertIn(self.node['replication_port'], cm.exception.args[0])
         self.assertEqual(self.node['replication_ip'], cm.exception.http_host)
         self.assertEqual(self.node['replication_port'], cm.exception.http_port)
         self.assertEqual(self.node['device'], cm.exception.http_device)
@@ -737,7 +774,7 @@ class TestDirectClient(unittest.TestCase):
         self._test_direct_get_suffix_hashes_fail(507)
 
     def test_direct_put_object_with_content_length(self):
-        contents = six.StringIO('123456')
+        contents = six.BytesIO(b'123456')
 
         with mocked_http_conn(200) as conn:
             resp = direct_client.direct_put_object(
@@ -747,28 +784,24 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'PUT')
             self.assertEqual(conn.path, self.obj_path)
-        self.assertEqual(md5('123456').hexdigest(), resp)
+        self.assertEqual(md5(b'123456').hexdigest(), resp)
 
     def test_direct_put_object_fail(self):
-        contents = six.StringIO('123456')
+        contents = six.BytesIO(b'123456')
 
         with mocked_http_conn(500) as conn:
-            try:
+            with self.assertRaises(ClientException) as raised:
                 direct_client.direct_put_object(
                     self.node, self.part, self.account, self.container,
                     self.obj, contents)
-            except ClientException as err:
-                pass
-            else:
-                self.fail('ClientException not raised')
             self.assertEqual(conn.host, self.node['ip'])
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'PUT')
             self.assertEqual(conn.path, self.obj_path)
-        self.assertEqual(err.http_status, 500)
+        self.assertEqual(raised.exception.http_status, 500)
 
     def test_direct_put_object_chunked(self):
-        contents = six.StringIO('123456')
+        contents = six.BytesIO(b'123456')
 
         with mocked_http_conn(200) as conn:
             resp = direct_client.direct_put_object(
@@ -778,7 +811,7 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual(conn.method, 'PUT')
             self.assertEqual(conn.path, self.obj_path)
-        self.assertEqual(md5('6\r\n123456\r\n0\r\n\r\n').hexdigest(), resp)
+        self.assertEqual(md5(b'6\r\n123456\r\n0\r\n\r\n').hexdigest(), resp)
 
     def test_direct_put_object_args(self):
         # One test to cover all missing checks
@@ -793,10 +826,10 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(self.obj_path, conn.path)
             self.assertEqual(conn.req_headers['Content-Length'], '0')
             self.assertEqual(conn.req_headers['Content-Type'], 'Text')
-        self.assertEqual(md5('0\r\n\r\n').hexdigest(), resp)
+        self.assertEqual(md5(b'0\r\n\r\n').hexdigest(), resp)
 
     def test_direct_put_object_header_content_length(self):
-        contents = six.StringIO('123456')
+        contents = six.BytesIO(b'123456')
         stub_headers = HeaderKeyDict({
             'Content-Length': '6'})
 
@@ -808,7 +841,7 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.port, self.node['port'])
             self.assertEqual('PUT', conn.method)
             self.assertEqual(conn.req_headers['Content-length'], '6')
-        self.assertEqual(md5('123456').hexdigest(), resp)
+        self.assertEqual(md5(b'123456').hexdigest(), resp)
 
     def test_retry(self):
         headers = HeaderKeyDict({'key': 'value'})
@@ -835,14 +868,11 @@ class TestDirectClient(unittest.TestCase):
                                     retries=2, error_log=logger.error)
         self.assertEqual('DELETE', conn.method)
         self.assertEqual(err_ctx.exception.http_status, 500)
-        self.assertIn('DELETE', err_ctx.exception.message)
-        self.assertIn(quote('/%s/%s/%s/%s/%s'
-                            % (self.node['device'].encode('utf-8'),
-                               self.part, self.account,
-                               self.container, self.obj)),
-                      err_ctx.exception.message)
-        self.assertIn(self.node['ip'], err_ctx.exception.message)
-        self.assertIn(self.node['port'], err_ctx.exception.message)
+        self.assertIn('DELETE', err_ctx.exception.args[0])
+        self.assertIn(self.obj_path,
+                      err_ctx.exception.args[0])
+        self.assertIn(self.node['ip'], err_ctx.exception.args[0])
+        self.assertIn(self.node['port'], err_ctx.exception.args[0])
         self.assertEqual(self.node['ip'], err_ctx.exception.http_host)
         self.assertEqual(self.node['port'], err_ctx.exception.http_port)
         self.assertEqual(self.node['device'], err_ctx.exception.http_device)
