@@ -20,6 +20,7 @@ import os
 import random
 from tempfile import mkdtemp
 from shutil import rmtree
+from eventlet import Timeout
 
 from swift.common.utils import normalize_timestamp
 from swift.container import auditor
@@ -59,8 +60,9 @@ class TestAuditor(unittest.TestCase):
     def tearDown(self):
         rmtree(os.path.dirname(self.testdir), ignore_errors=1)
 
+    @mock.patch('swift.container.auditor.dump_recon_cache')
     @mock.patch('swift.container.auditor.ContainerBroker', FakeContainerBroker)
-    def test_run_forever(self):
+    def test_run_forever(self, mock_recon):
         sleep_times = random.randint(5, 10)
         call_times = sleep_times - 1
 
@@ -93,8 +95,15 @@ class TestAuditor(unittest.TestCase):
         self.assertEqual(test_auditor.container_failures, 2 * call_times)
         self.assertEqual(test_auditor.container_passes, 3 * call_times)
 
+        # now force timeout path code coverage
+        with mock.patch('swift.container.auditor.ContainerAuditor.'
+                        '_one_audit_pass', side_effect=Timeout()):
+            with mock.patch('swift.container.auditor.time', FakeTime()):
+                self.assertRaises(ValueError, test_auditor.run_forever)
+
+    @mock.patch('swift.container.auditor.dump_recon_cache')
     @mock.patch('swift.container.auditor.ContainerBroker', FakeContainerBroker)
-    def test_run_once(self):
+    def test_run_once(self, mock_recon):
         conf = {}
         test_auditor = auditor.ContainerAuditor(conf, logger=self.logger)
 
@@ -108,6 +117,24 @@ class TestAuditor(unittest.TestCase):
         self.assertEqual(test_auditor.container_failures, 2)
         self.assertEqual(test_auditor.container_passes, 3)
 
+    @mock.patch('swift.container.auditor.dump_recon_cache')
+    @mock.patch('swift.container.auditor.ContainerBroker', FakeContainerBroker)
+    def test_one_audit_pass(self, mock_recon):
+        conf = {}
+        test_auditor = auditor.ContainerAuditor(conf, logger=self.logger)
+
+        def fake_audit_location_generator(*args, **kwargs):
+            files = sorted(os.listdir(self.testdir))
+            return [(os.path.join(self.testdir, f), '', '') for f in files]
+
+        # force code coverage for logging path
+        test_auditor.logging_interval = 0
+        with mock.patch('swift.container.auditor.audit_location_generator',
+                        fake_audit_location_generator):
+            test_auditor._one_audit_pass(test_auditor.logging_interval)
+        self.assertEqual(test_auditor.container_failures, 1)
+        self.assertEqual(test_auditor.container_passes, 3)
+
     @mock.patch('swift.container.auditor.ContainerBroker', FakeContainerBroker)
     def test_container_auditor(self):
         conf = {}
@@ -123,7 +150,8 @@ class TestAuditor(unittest.TestCase):
 class TestAuditorMigrations(unittest.TestCase):
 
     @with_tempdir
-    def test_db_migration(self, tempdir):
+    @mock.patch('swift.container.auditor.dump_recon_cache')
+    def test_db_migration(self, tempdir, mock_recon):
         db_path = os.path.join(tempdir, 'sda', 'containers', '0', '0', '0',
                                'test.db')
         with test_backend.TestContainerBrokerBeforeSPI.old_broker() as \
