@@ -14,20 +14,27 @@
 # limitations under the License.
 
 import itertools
+import json
 from collections import defaultdict
 import unittest
 import mock
+
+import six
+
 from swift.proxy.controllers.base import headers_to_container_info, \
     headers_to_account_info, headers_to_object_info, get_container_info, \
     get_cache_key, get_account_info, get_info, get_object_info, \
     Controller, GetOrHeadHandler, bytes_to_skip
 from swift.common.swob import Request, HTTPException, RESPONSE_REASONS
 from swift.common import exceptions
-from swift.common.utils import split_path
+from swift.common.utils import split_path, ShardRange, Timestamp
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.http import is_success
 from swift.common.storage_policy import StoragePolicy, StoragePolicyCollection
-from test.unit import fake_http_connect, FakeRing, FakeMemcache, PatchPolicies
+from test.unit import (
+    fake_http_connect, FakeRing, FakeMemcache, PatchPolicies, FakeLogger,
+    make_timestamp_iter,
+    mocked_http_conn)
 from swift.proxy import server as proxy_server
 from swift.common.request_helpers import (
     get_sys_meta_prefix, get_object_transient_sysmeta
@@ -40,7 +47,7 @@ class FakeResponse(object):
 
     base_headers = {}
 
-    def __init__(self, status_int=200, headers=None, body=''):
+    def __init__(self, status_int=200, headers=None, body=b''):
         self.status_int = status_int
         self._headers = headers or {}
         self.body = body
@@ -172,7 +179,8 @@ class TestFuncs(unittest.TestCase):
     def setUp(self):
         self.app = proxy_server.Application(None, FakeMemcache(),
                                             account_ring=FakeRing(),
-                                            container_ring=FakeRing())
+                                            container_ring=FakeRing(),
+                                            logger=FakeLogger())
 
     def test_get_info_zero_recheck(self):
         mock_cache = mock.Mock()
@@ -370,7 +378,10 @@ class TestFuncs(unittest.TestCase):
         self.assertEqual(resp['bytes'], 3333)
         self.assertEqual(resp['object_count'], 10)
         self.assertEqual(resp['status'], 404)
-        self.assertEqual(resp['versions'], "\xe1\xbd\x8a\x39")
+        if six.PY3:
+            self.assertEqual(resp['versions'], u'\u1f4a9')
+        else:
+            self.assertEqual(resp['versions'], "\xe1\xbd\x8a\x39")
 
     def test_get_container_info_env(self):
         cache_key = get_cache_key("account", "cont")
@@ -700,9 +711,9 @@ class TestFuncs(unittest.TestCase):
     def test_best_response_overrides(self):
         base = Controller(self.app)
         responses = [
-            (302, 'Found', '', 'The resource has moved temporarily.'),
-            (100, 'Continue', '', ''),
-            (404, 'Not Found', '', 'Custom body'),
+            (302, 'Found', '', b'The resource has moved temporarily.'),
+            (100, 'Continue', '', b''),
+            (404, 'Not Found', '', b'Custom body'),
         ]
         server_type = "Base DELETE"
         req = Request.blank('/v1/a/c/o', method='DELETE')
@@ -721,7 +732,7 @@ class TestFuncs(unittest.TestCase):
         resp = base.best_response(req, statuses, reasons, bodies, server_type,
                                   headers=headers, overrides=overrides)
         self.assertEqual(resp.status, '404 Not Found')
-        self.assertEqual(resp.body, 'Custom body')
+        self.assertEqual(resp.body, b'Custom body')
 
     def test_range_fast_forward(self):
         req = Request.blank('/')
@@ -849,7 +860,7 @@ class TestFuncs(unittest.TestCase):
                 if self.chunks:
                     return self.chunks.pop(0)
                 else:
-                    return ''
+                    return b''
 
             def getheader(self, header):
                 if header.lower() == "content-length":
@@ -859,7 +870,7 @@ class TestFuncs(unittest.TestCase):
                 return [('content-length', self.getheader('content-length'))]
 
         source = TestSource((
-            'abcd', '1234', 'abc', 'd1', '234abcd1234abcd1', '2'))
+            b'abcd', b'1234', b'abc', b'd1', b'234abcd1234abcd1', b'2'))
         req = Request.blank('/v1/a/c/o')
         node = {}
         handler = GetOrHeadHandler(self.app, req, None, None, None, None, {},
@@ -868,7 +879,7 @@ class TestFuncs(unittest.TestCase):
         app_iter = handler._make_app_iter(req, node, source)
         client_chunks = list(app_iter)
         self.assertEqual(client_chunks, [
-            'abcd1234', 'abcd1234', 'abcd1234', 'abcd12'])
+            b'abcd1234', b'abcd1234', b'abcd1234', b'abcd12'])
 
     def test_client_chunk_size_resuming(self):
 
@@ -885,7 +896,7 @@ class TestFuncs(unittest.TestCase):
                     else:
                         return chunk
                 else:
-                    return ''
+                    return b''
 
             def getheader(self, header):
                 # content-length for the whole object is generated dynamically
@@ -899,11 +910,11 @@ class TestFuncs(unittest.TestCase):
 
         node = {'ip': '1.2.3.4', 'port': 6200, 'device': 'sda'}
 
-        source1 = TestSource(['abcd', '1234', None,
-                              'efgh', '5678', 'lots', 'more', 'data'])
+        source1 = TestSource([b'abcd', b'1234', None,
+                              b'efgh', b'5678', b'lots', b'more', b'data'])
         # incomplete reads of client_chunk_size will be re-fetched
-        source2 = TestSource(['efgh', '5678', 'lots', None])
-        source3 = TestSource(['lots', 'more', 'data'])
+        source2 = TestSource([b'efgh', b'5678', b'lots', None])
+        source3 = TestSource([b'lots', b'more', b'data'])
         req = Request.blank('/v1/a/c/o')
         handler = GetOrHeadHandler(
             self.app, req, 'Object', None, None, None, {},
@@ -922,7 +933,7 @@ class TestFuncs(unittest.TestCase):
             client_chunks = list(app_iter)
         self.assertEqual(range_headers, ['bytes=8-27', 'bytes=16-27'])
         self.assertEqual(client_chunks, [
-            'abcd1234', 'efgh5678', 'lotsmore', 'data'])
+            b'abcd1234', b'efgh5678', b'lotsmore', b'data'])
 
     def test_client_chunk_size_resuming_chunked(self):
 
@@ -941,7 +952,7 @@ class TestFuncs(unittest.TestCase):
                     else:
                         return chunk
                 else:
-                    return ''
+                    return b''
 
             def getheader(self, header):
                 return self.headers.get(header.lower())
@@ -951,8 +962,8 @@ class TestFuncs(unittest.TestCase):
 
         node = {'ip': '1.2.3.4', 'port': 6200, 'device': 'sda'}
 
-        source1 = TestChunkedSource(['abcd', '1234', 'abc', None])
-        source2 = TestChunkedSource(['efgh5678'])
+        source1 = TestChunkedSource([b'abcd', b'1234', b'abc', None])
+        source2 = TestChunkedSource([b'efgh5678'])
         req = Request.blank('/v1/a/c/o')
         handler = GetOrHeadHandler(
             self.app, req, 'Object', None, None, None, {},
@@ -962,7 +973,7 @@ class TestFuncs(unittest.TestCase):
         with mock.patch.object(handler, '_get_source_and_node',
                                lambda: (source2, node)):
             client_chunks = list(app_iter)
-        self.assertEqual(client_chunks, ['abcd1234', 'efgh5678'])
+        self.assertEqual(client_chunks, [b'abcd1234', b'efgh5678'])
 
     def test_disconnected_warning(self):
         self.app.logger = mock.Mock()
@@ -975,7 +986,7 @@ class TestFuncs(unittest.TestCase):
                 self.status = 200
 
             def read(self, _read_size):
-                return 'the cake is a lie'
+                return b'the cake is a lie'
 
             def getheader(self, header):
                 return self.headers.get(header.lower())
@@ -1030,3 +1041,154 @@ class TestFuncs(unittest.TestCase):
         # prime numbers
         self.assertEqual(bytes_to_skip(11, 7), 4)
         self.assertEqual(bytes_to_skip(97, 7873823), 55)
+
+    def test_get_shard_ranges_for_container_get(self):
+        ts_iter = make_timestamp_iter()
+        shard_ranges = [dict(ShardRange(
+            '.sharded_a/sr%d' % i, next(ts_iter), '%d_lower' % i,
+            '%d_upper' % i, object_count=i, bytes_used=1024 * i,
+            meta_timestamp=next(ts_iter)))
+            for i in range(3)]
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c', method='GET')
+        resp_headers = {'X-Backend-Record-Type': 'shard'}
+        with mocked_http_conn(
+            200, 200,
+            body_iter=iter([b'', json.dumps(shard_ranges).encode('ascii')]),
+            headers=resp_headers
+        ) as fake_conn:
+            actual = base._get_shard_ranges(req, 'a', 'c')
+
+        # account info
+        captured = fake_conn.requests
+        self.assertEqual('HEAD', captured[0]['method'])
+        self.assertEqual('a', captured[0]['path'][7:])
+        # container GET
+        self.assertEqual('GET', captured[1]['method'])
+        self.assertEqual('a/c', captured[1]['path'][7:])
+        self.assertEqual('format=json', captured[1]['qs'])
+        self.assertEqual(
+            'shard', captured[1]['headers'].get('X-Backend-Record-Type'))
+        self.assertEqual(shard_ranges, [dict(pr) for pr in actual])
+        self.assertFalse(self.app.logger.get_lines_for_level('error'))
+
+    def test_get_shard_ranges_for_object_put(self):
+        ts_iter = make_timestamp_iter()
+        shard_ranges = [dict(ShardRange(
+            '.sharded_a/sr%d' % i, next(ts_iter), '%d_lower' % i,
+            '%d_upper' % i, object_count=i, bytes_used=1024 * i,
+            meta_timestamp=next(ts_iter)))
+            for i in range(3)]
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c/o', method='PUT')
+        resp_headers = {'X-Backend-Record-Type': 'shard'}
+        with mocked_http_conn(
+            200, 200,
+            body_iter=iter([b'',
+                            json.dumps(shard_ranges[1:2]).encode('ascii')]),
+            headers=resp_headers
+        ) as fake_conn:
+            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+
+        # account info
+        captured = fake_conn.requests
+        self.assertEqual('HEAD', captured[0]['method'])
+        self.assertEqual('a', captured[0]['path'][7:])
+        # container GET
+        self.assertEqual('GET', captured[1]['method'])
+        self.assertEqual('a/c', captured[1]['path'][7:])
+        params = sorted(captured[1]['qs'].split('&'))
+        self.assertEqual(
+            ['format=json', 'includes=1_test'], params)
+        self.assertEqual(
+            'shard', captured[1]['headers'].get('X-Backend-Record-Type'))
+        self.assertEqual(shard_ranges[1:2], [dict(pr) for pr in actual])
+        self.assertFalse(self.app.logger.get_lines_for_level('error'))
+
+    def _check_get_shard_ranges_bad_data(self, body):
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c/o', method='PUT')
+        # empty response
+        headers = {'X-Backend-Record-Type': 'shard'}
+        with mocked_http_conn(200, 200, body_iter=iter([b'', body]),
+                              headers=headers):
+            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertIsNone(actual)
+        lines = self.app.logger.get_lines_for_level('error')
+        return lines
+
+    def test_get_shard_ranges_empty_body(self):
+        error_lines = self._check_get_shard_ranges_bad_data(b'')
+        self.assertIn('Problem with listing response', error_lines[0])
+        if six.PY2:
+            self.assertIn('No JSON', error_lines[0])
+        else:
+            self.assertIn('JSONDecodeError', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_not_a_list(self):
+        body = json.dumps({}).encode('ascii')
+        error_lines = self._check_get_shard_ranges_bad_data(body)
+        self.assertIn('Problem with listing response', error_lines[0])
+        self.assertIn('not a list', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_key_missing(self):
+        body = json.dumps([{}]).encode('ascii')
+        error_lines = self._check_get_shard_ranges_bad_data(body)
+        self.assertIn('Failed to get shard ranges', error_lines[0])
+        self.assertIn('KeyError', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_invalid_shard_range(self):
+        sr = ShardRange('a/c', Timestamp.now())
+        bad_sr_data = dict(sr, name='bad_name')
+        body = json.dumps([bad_sr_data]).encode('ascii')
+        error_lines = self._check_get_shard_ranges_bad_data(body)
+        self.assertIn('Failed to get shard ranges', error_lines[0])
+        self.assertIn('ValueError', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_missing_record_type(self):
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c/o', method='PUT')
+        sr = ShardRange('a/c', Timestamp.now())
+        body = json.dumps([dict(sr)]).encode('ascii')
+        with mocked_http_conn(
+                200, 200, body_iter=iter([b'', body])):
+            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertIsNone(actual)
+        error_lines = self.app.logger.get_lines_for_level('error')
+        self.assertIn('Failed to get shard ranges', error_lines[0])
+        self.assertIn('unexpected record type', error_lines[0])
+        self.assertIn('/a/c', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_wrong_record_type(self):
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c/o', method='PUT')
+        sr = ShardRange('a/c', Timestamp.now())
+        body = json.dumps([dict(sr)]).encode('ascii')
+        headers = {'X-Backend-Record-Type': 'object'}
+        with mocked_http_conn(
+                200, 200, body_iter=iter([b'', body]),
+                headers=headers):
+            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertIsNone(actual)
+        error_lines = self.app.logger.get_lines_for_level('error')
+        self.assertIn('Failed to get shard ranges', error_lines[0])
+        self.assertIn('unexpected record type', error_lines[0])
+        self.assertIn('/a/c', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_request_failed(self):
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c/o', method='PUT')
+        with mocked_http_conn(200, 404, 404, 404):
+            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertIsNone(actual)
+        self.assertFalse(self.app.logger.get_lines_for_level('error'))
+        warning_lines = self.app.logger.get_lines_for_level('warning')
+        self.assertIn('Failed to get container listing', warning_lines[0])
+        self.assertIn('/a/c', warning_lines[0])
+        self.assertFalse(warning_lines[1:])
